# pylint: disable=too-many-branches,too-many-return-statements
#  Drakkar-Software OctoBot-Commons
#  Copyright (c) Drakkar-Software, All rights reserved.
#
#  This library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public
#  License as published by the Free Software Foundation; either
#  version 3.0 of the License, or (at your option) any later version.
#
#  This library is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this library.
import dataclasses
import typing


@dataclasses.dataclass
class OperatorParameter:
    name: str
    description: str
    required: bool
    type: typing.Type[typing.Any]

    def __repr__(self) -> str:
        return f"{self.name}{' (required)' if self.required else ''}[{self.type.__name__}] - {self.description}"

    def to_json(self) -> dict:
        """
        Convert the operator parameter to a JSON serializable dict.
        """
        return {
            "name": self.name,
            "description": self.description,
            "required": self.required,
            "type": self.type.__name__,
        }



#  Drakkar-Software OctoBot-Commons
#  Copyright (c) Drakkar-Software, All rights reserved.
#
#  This library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public
#  License as published by the Free Software Foundation; either
#  version 3.0 of the License, or (at your option) any later version.
#
#  This library is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this library.
import functools


def flatten_list(list_to_flatten):
    """
    Flatten the list :list_to_flatten:
    :param list_to_flatten: the list to flatten
    :return: the flattened list
    """
    return functools.reduce(
        lambda first_level, second_level: first_level + second_level, list_to_flatten
    )


def deduplicate(elements: list) -> list:
    """
    remove duplicated values from a list while preserving order
    """
    # from https://stackoverflow.com/questions/480214/how-do-i-remove-duplicates-from-a-list-while-preserving-order
    seen = set()
    seen_add = seen.add
    return [x for x in elements if not (x in seen or seen_add(x))]



#  Drakkar-Software OctoBot-Commons
#  Copyright (c) Drakkar-Software, All rights reserved.
#
#  This library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public
#  License as published by the Free Software Foundation; either
#  version 3.0 of the License, or (at your option) any later version.
#
#  This library is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this library.


from octobot_commons.profiles import profile

from octobot_commons.profiles.profile import (
    Profile,
)

from octobot_commons.profiles import profile_sharing
from octobot_commons.profiles.profile_sharing import (
    export_profile,
    install_profile,
    import_profile,
    import_profile_data_as_profile,
    update_profile,
    download_profile,
    download_and_install_profile,
)

from octobot_commons.profiles import profile_data

from octobot_commons.profiles.profile_data import (
    ProfileData,
    ExchangeData,
    MinimalFund,
    OptionsData,
)

from octobot_commons.profiles import profile_sync

from octobot_commons.profiles.profile_sync import (
    start_profile_synchronizer,
    stop_profile_synchronizer,
)

from octobot_commons.profiles import exchange_auth_data

from octobot_commons.profiles.exchange_auth_data import (
    ExchangeAuthData,
)

from octobot_commons.profiles import tentacles_profile_data_translator

from octobot_commons.profiles.tentacles_profile_data_translator import (
    TentaclesProfileDataTranslator,
)

from octobot_commons.profiles import tentacles_profile_data_adapter

from octobot_commons.profiles.tentacles_profile_data_adapter import (
    TentaclesProfileDataAdapter,
)


__all__ = [
    "Profile",
    "export_profile",
    "install_profile",
    "import_profile",
    "import_profile_data_as_profile",
    "update_profile",
    "download_profile",
    "download_and_install_profile",
    "ProfileData",
    "ExchangeData",
    "MinimalFund",
    "OptionsData",
    "start_profile_synchronizer",
    "stop_profile_synchronizer",
    "TentaclesProfileDataTranslator",
    "TentaclesProfileDataAdapter",
    "ExchangeAuthData",
]
